﻿using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Runtime.InteropServices;
using System.Text;
using System.Windows.Forms;
using MDL.Draw.Chemistry;
using MDL.Draw.Chemistry.IO;
using MDL.Draw.HeadlessRenderer;
using MDL.Draw.Utils;


namespace ADEmbed {
    class Program {
        static void Main(string[] args) {
            String molIn = args[0];
            String contentsOut = args[1];
            String emfOut = args[2];
            int width = Convert.ToInt32(args[3]);
            int height = Convert.ToInt32(args[4]);

            MoleculeIO m_MoleculeIO = new MoleculeIO();
            Molecule molecule = m_MoleculeIO.ReadMolecule(molIn, 1);
            // Fragment.PerceiveFragments(molecule);

            HeadlessRenderer m_HeadlessRenderer = new HeadlessRenderer();
            m_HeadlessRenderer.Preferences.ForeColor = Color.Black;
            m_HeadlessRenderer.Preferences.BackColor = Color.Empty;

            Form m_dummyForm = new Form();
            System.Drawing.Graphics graphics = m_dummyForm.CreateGraphics();

            IntPtr hdc = graphics.GetHdc();
            String molStr = MoleculeIO.GenerateMolfileString(molecule);
            Metafile mfEmf = m_HeadlessRenderer.PaintMolecule(molecule, StructureType.StructureHandle, hdc, width, height);
            Metafile mfWmf = (Metafile)mfEmf.Clone();

            byte[] molRlenBytes = null;
            if (!molecule.ContainsMultipleReactionArrows) {
                molRlenBytes = FileToBufferedFile(molStr).ToArray();
            }

            byte[] skcBytes = null;
            if (DrawUtil.SketchEnabled) {
                MoleculePrefs mp = new MoleculePrefs();
                skcBytes = SkcWriter.GenerateSkcStream(molecule).ToArray();
            }

            byte[] emfBytes = GetEnhancedMetafileBytes(mfEmf);
            byte[] wmfBytes = GetWindowsMetafileBytes(mfWmf);
            byte[] molBytes = new ASCIIEncoding().GetBytes(molStr);

            graphics.ReleaseHdc(hdc);
            graphics.Dispose();
            mfWmf.Dispose();
            mfEmf.Dispose();

            FileStream fsEmf = new FileStream(emfOut, FileMode.Create, FileAccess.Write);
            fsEmf.Write(emfBytes, 0, emfBytes.Length);
            fsEmf.Close();

            FileStream fsContents = new FileStream(contentsOut, FileMode.Create, FileAccess.Write);
            addBytesToStream(fsContents, molBytes);
            addBytesToStream(fsContents, skcBytes);
            addBytesToStream(fsContents, molRlenBytes);
            addBytesToStream(fsContents, emfBytes);
            addBytesToStream(fsContents, wmfBytes);
            fsContents.Close();
        }

        public static void addBytesToStream(FileStream fs, byte[] bytes) {
            if (bytes == null) return;
            fs.Write(INT2LE(bytes.Length), 0, 4);
            fs.Write(bytes, 0, bytes.Length);
        }

        public static byte[] INT2LE(int data) {
            byte[] b = new byte[4];
            b[0] = (byte)data;
            b[1] = (byte)(((uint)data >> 8) & 0xFF);
            b[2] = (byte)(((uint)data >> 16) & 0xFF);
            b[3] = (byte)(((uint)data >> 24) & 0xFF);
            return b;
        }

        // convert molstring, so each line has the line length in front (max. 255 char len ...)
        public static MemoryStream FileToBufferedFile(string s) {
            int length = s.Length;
            MemoryStream output = new MemoryStream();
            BinaryWriter writer = new BinaryWriter(output);
            int startIndex = 0;
            int index = 0;
            while (startIndex < length) {
                startIndex = index;
                index = s.IndexOf('\n', startIndex);
                if (index == -1) {
                    writer.Write((byte)0);
                    writer.Close();
                    return output;
                }
                string str = s.Substring(startIndex, index++ - startIndex);
                writer.Write((byte)str.Length);
                if (str.Length > 0) {
                    byte[] bytes = Encoding.ASCII.GetBytes(str);
                    writer.Write(bytes);
                }
            }
            return output;
        }


        public static byte[] GetEnhancedMetafileBytes(Metafile mf) {
            try {
                IntPtr henhmetafile = mf.GetHenhmetafile();
                if (!henhmetafile.Equals(IntPtr.Zero)) {
                    uint cbBuffer = GetEnhMetaFileBits(henhmetafile, 0, null);
                    byte[] lpbBuffer = new byte[cbBuffer];
                    cbBuffer = GetEnhMetaFileBits(henhmetafile, cbBuffer, lpbBuffer);
                    return lpbBuffer;
                }
            }
            catch {
            }
            return null;
        }

        public static byte[] GetWindowsMetafileBytes(Metafile mf) {
            try {
                IntPtr henhmetafile = mf.GetHenhmetafile();
                if (!henhmetafile.Equals(IntPtr.Zero)) {
                    uint num = GdipEmfToWmfBits(henhmetafile, 0, null, 8, 0);
                    byte[] buffer = new byte[num];
                    num = GdipEmfToWmfBits(henhmetafile, num, buffer, 8, 0);
                    return buffer;
                }
            }
            catch {
            }
            return null;
        }

        [DllImport("gdi32.dll")]
        private static extern uint GetEnhMetaFileBits(IntPtr hemf, uint cbBuffer, byte[] lpbBuffer);
        [DllImport("gdiplus.dll")]
        private static extern uint GdipEmfToWmfBits(IntPtr _hEmf, uint _bufferSize, byte[] _buffer, int _mappingMode, int _flags);

    }
}
